require 'socket'

PATH_TO_SETS = ".."

class String
  def number?
    true if Float(self) rescue false
  end
end

class SetlistServer
  def initialize(filename)
    @sets = IO.readlines(filename)
              .compact
              .collect { |s| s.strip }
              .reject { |s| s.empty? }
    @udp = UDPSocket.new
    @udp.bind(nil, 7400)
  end
  
  def start
    load_set(0)
    listen
  end
  
  def listen
    loop do     
      command, sender = @udp.recvfrom(2)
      command.strip!
      puts "Received command #{command}"

      if command.number?
        load_set(command.to_i)
      else
        case command.chr.downcase
        when "n"
          load_set(@currSet + 1)
        when "p"
          load_set(@currSet - 1)
        when "r"
          load_set(@currSet)
        else
          puts "Unknown command"
        end
      end     
    end
  end

  def load_set(i)
    unless i.between?(0, @sets.length - 1)
      puts "Invalid command"
      return
    end

    s = @sets[i]
    print "Loading set #{s}..."
    system "open #{PATH_TO_SETS}/#{s}/#{s}.als"
    if $? == 0
      puts "set loaded"
      @currSet = i
    else
      puts "ERROR! Failed to load set"
    end
  end
end

ss = SetlistServer.new(ARGV[0])
ss.start